//=================================================
//
// Class			AudioApplet
//
// Author:			Grzegorz Lewandowski
// created			01/06/98
//
//-------------------------------------------------

import java.io.*;
import java.net.*;
import java.applet.*;
import java.awt.*;
import sun.audio.*;

public class AudioApplet extends Applet
{
	private String audioSource = "";

	// user interface
	private java.awt.Button buttonPlay;
	private java.awt.Button buttonStop;
	private java.awt.Button buttonRewind;
	private java.awt.Button buttonBrowse;
	private java.awt.TextField textField;

	private java.awt.Panel panelText;
	private java.awt.Panel panelButtons;

	private int state;

	// player state
	static final int NEW = 0;
	static final int READY = 1;
	static final int PLAYING = 2;
	static final int STOPPED = 3;

	// player components
	private Thread providerThread = null;
	private AudioProvider provider = null;
	private AudioIOStream stream = null;
	private ADPCMdecoder decoder = null;
	private AudioCheck check = null;
	private StreamController controller = null;

	private byte[] buffer = null;

	// browser interface
	private Navigator frame = null;
	private URL siteMap = null;
	private Image doc = null;
	private Image dir = null;
	private Image open = null;


	public AudioApplet()
	{
	}

	public String getAppletInfo()
	{
		return "Name: AudioApplet\r\n" +
		       "Author: Grzegorz Lewandowski\r\n" +
		       "Created with Microsoft Visual J++ Version 1.1";
	}

	public void init()
	{
		System.out.println( "init()" );

		setLayout( new GridLayout( 2, 1 ) );
		addNotify();
		panelText = new java.awt.Panel();
		add( panelText );

		panelText.setLayout( new GridLayout( 1, 1, 0, 0 ) );
		panelText.addNotify();
		textField = new TextField();
		panelText.add( textField );
		
		panelButtons = new java.awt.Panel();
		add( panelButtons );

		panelButtons.setLayout( new GridLayout( 1, 1, 0, 0 ) );
		panelButtons.addNotify();
		panelButtons.resize( insets().left + insets().right + 210,insets().top + insets().bottom + 30 );
		buttonPlay = new java.awt.Button( "Play" );
		buttonPlay.reshape( insets().left + 0, insets().top + 0, 70, 30 );
		panelButtons.add( buttonPlay );
		buttonPlay.disable();
		buttonStop = new java.awt.Button( "Stop" );
		buttonStop.reshape( insets().left + 73, insets().top + 0, 70, 30 );
		panelButtons.add( buttonStop );
		buttonStop.disable();
		buttonRewind = new java.awt.Button( "Rewind" );
		buttonRewind.reshape( insets().left + 146, insets().top + 0, 70, 30 );
		panelButtons.add( buttonRewind );
		buttonRewind.disable();
		buttonBrowse = new java.awt.Button( "Browse >>" );
		// buttonBrowse.reshape();
		panelButtons.add( buttonBrowse );

	}

	public void destroy()
	{
		super.destroy();
		System.out.println( "destroy()" );

		// kill audio player
		controller.eof();
		if( providerThread != null )
		{
			providerThread.stop();
			providerThread = null;
		}

	}

	public void paint( Graphics g )
	{
		// System.out.println( "paint()" );
	}

	public void start()
	{
		System.out.println( "start()" );
		DataInputStream dataInput = null;
		// open config file
		try
		{
			URL url = new URL( getDocumentBase(), "audio.cfg" );
			InputStream input = url.openStream();
			dataInput = new DataInputStream( input );
		}
		catch( Exception e )
		{
			e.printStackTrace( System.err );
		}
		String line;

		// browse config file
		try
		{
			while( ( line = dataInput.readLine() ) != null )
			{
				String tag = line.trim().substring( 0, line.trim().indexOf( ' ' ) ).trim();
				String data = line.trim().substring( line.trim().indexOf( ' ' ) ).trim();
				if( tag.compareTo( "defaultURL" ) == 0 )
				{
					System.out.println( "Default URL: " + data );
					if( data.compareTo( "none" ) != 0 )
					{
						textField.setText( data );
						urlEntered();
					}
				}

				if( tag.compareTo( "siteMap" ) == 0 )
				{
					System.out.println( "Site Map: " + data );
					try
					{
						siteMap = new URL( getDocumentBase(), data );
					}
					catch( Exception e )
					{
						e.printStackTrace( System.err );
					}
				}

				if( tag.compareTo( "doc" ) == 0 )
				{
					System.out.println( "Document Image: " + data );
					doc = getImage( getDocumentBase(), data );
				}

				if( tag.compareTo( "open" ) == 0 )
				{
					System.out.println( "Open Directory Image: " + data );
					open = getImage( getDocumentBase(), data );
				}

				if( tag.compareTo( "dir" ) == 0 )
				{
					System.out.println( "Dircetory Image: " + data );
					dir = getImage( getDocumentBase(), data );
				}
			}
		}
		catch( Exception e )
		{
			e.printStackTrace( System.err );
		}

		try
		{
			dataInput.close();
		}
		catch( Exception e )
		{
			e.printStackTrace( System.err );
		}
	}
	
	public void stop()
	{
		System.out.println( "stop()" );
	}

	public URL getSiteMap()
	{
		return siteMap;
	}

	public Image getDocImage()
	{
		return doc;
	}

	public Image getDirImage()
	{
		return dir;
	}

	public Image getOpenImage()
	{
		return open;
	}

	//--------------------------------------------------------------------------

	public boolean action( Event event, Object what )
	{
		if( event.target == buttonPlay )
		{
			buttonPlayClicked();
			return true;
		}
		if( event.target == buttonStop )
		{
			buttonStopClicked();
			return true;
		}
		if( event.target == buttonRewind )
		{
			buttonRewindClicked();
			return true;
		}
		if( event.target == buttonBrowse )
		{
			buttonBrowseClicked();
			return true;
		}
		if( event.target == textField )
		{
			urlEntered();
			return true;
		}
		return false;
	}

	void putNewURL( URL url )
	{
		textField.setText( url.toString() );
		frame.hide();
		urlEntered();
	}

	void urlEntered()
	{
		System.out.println( "urlEntered()" );

		if( state != NEW )
		{
			System.out.println( "reinitialization started" );
			buttonStopClicked();
			if( state == READY )
			{
				state = STOPPED;
			}
			audioSource = textField.getText();
			buttonRewindClicked();
		}
		else
		{
			System.out.println( "initialization started" );
			stream = new AudioIOStream();
			decoder = new ADPCMdecoder( 800 );

			audioSource = textField.getText();
			provider = new URLAudioProvider( decoder, stream, audioSource );

			providerThread = new Thread( provider, "AudioProvider" );
			providerThread.start();

			controller = new StreamController( stream );

			check = new AudioCheck( this, stream );
			check.start();
		
			state = READY;
			buttonPlay.enable();
			buttonPlay.requestFocus();
			System.out.println( "Player: READY" );
		}
	}
		
	void buttonPlayClicked()
	{
		System.out.println( "buttonPlayClicked()" );

		if( state == READY )
		{
			AudioPlayer.player.start( (InputStream)controller );
			controller.open();
			buttonPlay.disable();
			buttonStop.enable();
			buttonRewind.disable();
			state = PLAYING;
			System.out.println( "Player: PLAYING" );
		}
		if( state == STOPPED )
		{
			controller.open();
			buttonPlay.disable();
			buttonStop.enable();
			buttonRewind.disable();
			state = PLAYING;
			System.out.println( "Player: PLAYING" );
		}
	}

	void buttonStopClicked()
	{
		System.out.println( "buttonStopClicked()" );

		if( state == PLAYING )
		{
			controller.shut();
			buttonPlay.enable();
			buttonStop.disable();
			buttonRewind.enable();
			state = STOPPED;
			System.out.println( "Player: STOPPED" );
		}
	}

	void buttonRewindClicked()
	{
		System.out.println( "buttonRewindClicked()" );

		if( state == STOPPED )
		{
			// kill audio player
			controller.eof();

			// not really necessary
			try
			{
				Thread.sleep( 500 );
			}
			catch( Exception e )
			{
				e.printStackTrace( System.err );
			}

			// recreate objects
			stream = new AudioIOStream();
			decoder = new ADPCMdecoder( 800 );
			provider.close();
			
			provider = new URLAudioProvider( decoder, stream, audioSource );

			providerThread = new Thread( provider, "AudioProvider" );
			providerThread.start();

			controller = new StreamController( stream );

			// reset user interface
			buttonPlay.enable();
			buttonStop.disable();
			buttonRewind.disable();

			// ready to go !!
			state = READY;
			System.out.println( "Player: READY" );
		}
	}

	void buttonBrowseClicked()
	{
		System.out.println( "buttonBrowseClicked()" );
		if( frame == null )
		{
			showStatus( "Loading..." );
			frame = new Navigator( this );
			frame.pack ();
		}
		frame.show();
	}

	void audioFinished()
	{
		stream = new AudioIOStream();
		decoder = new ADPCMdecoder( 800 );
		provider.close();

		provider = new URLAudioProvider( decoder, stream, audioSource );
		providerThread = new Thread( provider, "AudioProvider" );
		providerThread.start();

		controller = new StreamController( stream );

		buttonPlay.enable();
		buttonStop.disable();
		buttonRewind.disable();
		state = READY;

		check = new AudioCheck( this, stream );
		check.start();
		System.out.println( "Player: READY" );
	}
}
